package weaver.interfaces.htsc.util;

import com.alibaba.fastjson.JSONArray;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import weaver.general.BaseBean;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;

public class HttpRequestUtil extends BaseBean{
    /**
     * 模拟请求
     *
     * @param url        资源地址
     * @param params    参数列表
     * @param encoding    编码
     * @return
     * @throws java.text.ParseException
     * @throws IOException
     */
    public String sendPost(String url, String params,String encoding){
        String body = "";

        //创建httpclient对象
        CloseableHttpClient client = HttpClients.createDefault();
        //创建post方式请求对象
        HttpPost httpPost = new HttpPost(url);

        //设置参数到请求对象中
        StringEntity strEntity = new StringEntity(params,encoding);//解决中文乱码问题
        strEntity.setContentEncoding(encoding);
        strEntity.setContentType("application/json");
        httpPost.setEntity(strEntity);

        //设置header信息
        //指定报文头【Content-type】、【User-Agent】
        httpPost.setHeader("Content-type", "application/json");
        httpPost.setHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");

        //执行请求操作，并拿到结果（同步阻塞）
        CloseableHttpResponse response = null;
        try {
            response = client.execute(httpPost);
            //获取结果实体
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                //按指定编码转换结果实体为String类型
                body = EntityUtils.toString(entity, encoding);
            }
            EntityUtils.consume(entity);
            //释放链接
            response.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return body;
    }


    /**
     * 向指定URL发送GET方法的请求
     *
     * @param url
     *            发送请求的URL
     * @param param
     *            请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return result 所代表远程资源的响应结果
     */
    public String sendGet(String url, String param) {
        String result = "";
        BufferedReader in = null;
        try {
            String urlNameString = url + "?" + param;
            URL realUrl = new URL(urlNameString); // 打开和URL之间的连接
            URLConnection connection = realUrl.openConnection(); // 设置通用的请求属性
            connection.setRequestProperty("accept", "*/*");
            connection.setRequestProperty("connection", "Keep-Alive");
            connection.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 建立实际的连接
            connection.connect();
            // 获取所有响应头字段
            //Map<String, List<String>> map = connection.getHeaderFields();
            // 遍历所有的响应头字段
            //for (String key : map.keySet()) {
            //    System.out.println(key + "--->" + map.get(key));
            //}
            // 定义 BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream(),"UTF-8"));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送GET请求出现异常！" + e);
            e.printStackTrace();
        }
        // 使用finally块来关闭输入流
        finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }

        return result;
    }


    public static void main(String[] args) {

        HttpRequestUtil httpRequestUtil = new HttpRequestUtil();
        //String url = "http://168.61.69.225:8080/jtxt/uum/department/getAllDepartments";

        int page = 0;
        boolean hasNext = true;
        while(hasNext){
            String url = "http://168.61.69.225:8080/jtxt/uum/user/getAllEmployee?page="+page+"&size=50";
            String response = httpRequestUtil.sendPost(url, "","UTF-8");
            System.out.println(page);
            JSONArray userArray = JSONArray.parseArray(response);
            //System.out.println("size:"+userArray.size());
            if(userArray.size()<50){
                hasNext = false;
            }
            page++;
        }

//        String url = "http://168.61.69.225:8080/jtxt/uum/department/getIncreaseDepartments";
//        Map<String,String> parmasMap = new HashMap<String,String>();
//        parmasMap.put("createTimeGreater","1577858449");
//        String params = JSONObject.toJSONString(parmasMap);
//        String response = httpRequestUtil.sendPost(url, params,"UTF-8");
//        System.out.println(response);
//
//        System.out.println(TimeUtil.getCurrentDateString());
//        System.out.println(TimeUtil.getCurrentTimeString());


//        for(int i=0;i<deptArray.size();i++){
//            JSONObject deptJson = deptArray.getJSONObject(i);
//            String deptCode = deptJson.getString("deptCode");
//            String deptHrBP = Util.null2String(deptJson.getString("deptHrBP"));
//            System.out.println("deptcode:"+deptCode+"  deptHrBP:"+deptHrBP);
//
//        }


        //System.out.println(response);


    }


}